EXEC [dbo].[__BuildMaster_AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_GetVariableValues',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Variable_Values',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the variable values for the specified scope.'
GO

IF OBJECT_ID('[Variables_GetVariableValues]') IS NOT NULL DROP PROCEDURE [Variables_GetVariableValues]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_GetVariableValues]
(
    @Environment_Id INT,
    @ServerRole_Id INT,
    @Server_Id INT,
    @ApplicationGroup_Id INT,
    @Application_Id INT,
    @Deployable_Id INT,
    @Release_Number RELEASENUMBER,
    @Build_Number BUILDNUMBER,
	@Promotion_Id INT,
    @Execution_Id INT
)
AS
BEGIN

    SET NOCOUNT ON

    IF @Execution_Id IS NOT NULL
    BEGIN
		-- Use standard function for execution-scoped variable cascading
		SELECT *
		  FROM dbo.Variables_GetValuesForExecution(@Execution_Id, @Server_Id, @ServerRole_Id, @Deployable_Id)
    END
    ELSE
    BEGIN

		-- If deployable is specified but not application, find the application
		IF @Deployable_Id IS NOT NULL AND @Application_Id IS NULL
		BEGIN
			SET @Application_Id = (SELECT [Application_Id]
			                         FROM [Deployables]
			                        WHERE [Deployable_Id] = @Deployable_Id)
		END

		-- If application is specified but not application group find the group
		IF @Application_Id IS NOT NULL AND @ApplicationGroup_Id IS NULL
		BEGIN
			SET @ApplicationGroup_Id = (SELECT [ApplicationGroup_Id]
			                              FROM [Applications]
			                             WHERE [Application_Id] = @Application_Id)
		END

		-- Make sure irrelevant release number is not considered
		IF @Release_Number IS NOT NULL AND @Application_Id IS NULL
			SET @Release_Number = NULL

		-- Make sure irrelevant build number is not considered
		IF @Build_Number IS NOT NULL AND @Release_Number IS NULL
			SET @Build_Number = NULL

		DECLARE @Release_Id INT
		DECLARE @Build_Id INT

		IF @Promotion_Id IS NOT NULL
		BEGIN

			SELECT @Release_Id = [Release_Id],
			       @Build_Id = [Build_Id],
				   @Application_Id = [Application_Id]
			  FROM [BuildPromotions_Extended]
			 WHERE [Promotion_Id] = @Promotion_Id

		END
		ELSE
		BEGIN

		 SELECT @Release_Id = [Release_Id]
		   FROM [Releases]
		  WHERE [Application_Id] = @Application_Id
		    AND [Release_Number] = @Release_Number

		 SELECT @Build_Id = [Build_Id]
		   FROM [Builds_Extended]
		  WHERE [Application_Id] = @Application_Id
		    AND [Release_Number] = @Release_Number
			AND [Build_Number] = @Build_Number
		END

		-- Determine the narrowest scope that will be considered			
		DECLARE @DesiredScope_Code CHAR(1)
		    SET @DesiredScope_Code = CASE
				WHEN @Promotion_Id IS NOT NULL THEN 'P'
				WHEN @Build_Number IS NOT NULL THEN 'B'
				WHEN @Release_Number IS NOT NULL THEN 'R'
				WHEN @Deployable_Id IS NOT NULL THEN 'D'
				WHEN @Application_Id IS NOT NULL THEN 'A'
				WHEN @ApplicationGroup_Id IS NOT NULL THEN 'G'
				WHEN @Server_Id IS NOT NULL THEN 'V'
                WHEN @ServerRole_Id IS NOT NULL THEN 'L'
				WHEN @Environment_Id IS NOT NULL THEN 'E'
				ELSE 'S'
			END

		DECLARE @DesiredScope_Index INT
		    SET @DesiredScope_Index = CHARINDEX(@DesiredScope_Code, 'SELVGADRBPX')

		--Return the results by joining against all variable names in the system and filtering based on input criteria
		--The inner SELECT DISTINCTS required that the VDEF and VDEC tables need an index on the Variable_Name columns for decent performance
        SELECT [Variable_Name],
               [Value_Text],
               [Scope_Code],
               [Sensitive_Indicator]

          FROM (SELECT VN.[Variable_Name],
                       [Value_Text] = COALESCE(VDEF.[Value_Text], VDEC.[DefaultValue_Text]),
                       [Scope_Code] = COALESCE(VDEF.[Scope_Code], VDEC.[Scope_Code]),
                       [Sensitive_Indicator] = COALESCE(VDEF.[Sensitive_Indicator], VDEC.[Sensitive_Indicator]),
                       [Rank] = RANK() OVER (PARTITION BY VN.[Variable_Name] ORDER BY CHARINDEX(COALESCE(VDEF.[Scope_Code], VDEC.[Scope_Code]), 'SELVGADRBPX') DESC)
                  FROM (SELECT DISTINCT [Variable_Name]
						  FROM [VariableDeclarations]
                               UNION
                        SELECT DISTINCT [Variable_Name]
                          FROM [VariableDefinitions]) VN

					   LEFT JOIN [BuildPromotions] BP
					          ON BP.[Promotion_Id] = @Promotion_Id

                       LEFT JOIN [VariableDefinitions] VDEF
                              ON VN.[Variable_Name] = VDEF.[Variable_Name]
                             AND CHARINDEX(VDEF.[Scope_Code], 'SELVGADRBPX') <= @DesiredScope_Index
                             AND (VDEF.[Application_Id] IS NULL OR VDEF.[Application_Id] = @Application_Id)
                             AND (VDEF.[Release_Id] IS NULL OR (VDEF.[Release_Id] = @Release_Id))
                             AND (VDEF.[Build_Id] IS NULL OR (VDEF.[Build_Id] = @Build_Id))
                             AND (VDEF.[ApplicationGroup_Id] IS NULL OR VDEF.[ApplicationGroup_Id] = (
                                                                                                    SELECT TOP 1 AGA.[ApplicationGroup_Id]
                                                                                                      FROM dbo.ApplicationGroups_GetAncestorGroups(@ApplicationGroup_Id) AGA
                                                                                                     INNER JOIN [VariableDefinitions] VDI
                                                                                                             ON VDI.[ApplicationGroup_Id] = AGA.[ApplicationGroup_Id]
                                                                                                     WHERE VDI.[Variable_Name] = VDEF.[Variable_Name]
                                                                                                     ORDER BY AGA.[Level] ASC
                                                                                                      ))
                             AND (VDEF.[Deployable_Id] IS NULL OR VDEF.[Deployable_Id] = @Deployable_Id)
                             AND (VDEF.[Environment_Id] IS NULL OR VDEF.[Environment_Id] = @Environment_Id)
                             AND (VDEF.[Server_Id] IS NULL OR VDEF.[Server_Id] = @Server_Id)
                             AND (VDEF.[ServerRole_Id] IS NULL OR VDEF.[ServerRole_Id] = @ServerRole_Id)
							 AND (VDEF.[Promotion_Id] IS NULL OR VDEF.[Promotion_Id] = @Promotion_Id)
                       LEFT JOIN [VariableDeclarations] VDEC
                              ON VN.[Variable_Name] = VDEC.[Variable_Name]
                             AND CHARINDEX(VDEC.[Scope_Code], 'SELVGADRBPX') <= @DesiredScope_Index
                             AND (VDEC.[Scoped_Application_Id] IS NULL OR VDEC.[Scoped_Application_Id] = @Application_Id)
                             AND (VDEC.[Scoped_PipelineStage_Name] IS NULL OR (VDEC.[Scoped_Application_Id] = @Application_Id AND VDEC.[Scoped_PipelineStage_Name] = BP.[PipelineStage_Name]))
                 WHERE VDEF.[Scope_Code] = VDEC.[Scope_Code]
                    OR (VDEF.[Scope_Code] IS NULL AND VDEC.[Scope_Code] IS NOT NULL)
                    OR (VDEC.[Scope_Code] IS NULL AND VDEF.[Scope_Code] IS NOT NULL)) _
         WHERE [Rank] = 1

    END

END
GO

GRANT EXECUTE ON [Variables_GetVariableValues] TO [BuildMasterUser_Role]